@@ -9,36 +9,52 @@
 # specific language governing permissions and limitations under the License.
 #
 
+import logging
 import os
 import platform
 import subprocess
 
+log = logging.getLogger('RHN')
+log.setLevel(logging.WARNING)
+
 
 def _pv_linux_machine(machine):
     if machine == 'x86_64':
         return machine
     elif machine == 'aarch64':
         arch_info = '-' + machine
-    else:
+    elif machine == 'armv7l':
         arch_info = ''
-
-    cpu_info = subprocess.check_output(['lscpu']).decode()
-    model_info = [x for x in cpu_info.split('\n') if 'Model name' in x][0].split(' ')[-1]
-
-    if 'ARM1176' == model_info: 
-        return 'arm11' + arch_info
-    elif 'Cortex-A7' == model_info:
-        return 'cortex-a7' + arch_info
-    elif 'Cortex-A53' == model_info:
-        return 'cortex-a53' + arch_info
-    elif 'Cortex-A57' == model_info:
-        return 'cortex-a57' + arch_info
-    elif 'Cortex-A72' == model_info:
-        return 'cortex-a72' + arch_info
-    elif 'Cortex-A8' == model_info:
-        return 'beaglebone' + arch_info
     else:
-        raise NotImplementedError('Unsupported CPU.')
+        raise NotImplementedError("Unsupported CPU architecture: '%s'" % machine)
+
+    cpu_info = ''
+    try:
+        cpu_info = subprocess.check_output(['cat', '/proc/cpuinfo']).decode()
+        cpu_part_list = [x for x in cpu_info.split('\n') if 'CPU part' in x]
+        cpu_part = cpu_part_list[0].split(' ')[-1].lower()
+    except Exception as error:
+        raise RuntimeError("Failed to identify the CPU with '%s'\nCPU info: %s" % (error, cpu_info))
+
+    if '0xb76' == cpu_part:
+        return 'arm11'
+    elif '0xc07' == cpu_part:
+        return 'cortex-a7'
+    elif '0xd03' == cpu_part:
+        return 'cortex-a53' + arch_info
+    elif '0xd07' == cpu_part:
+        return 'cortex-a57' + arch_info
+    elif '0xd08' == cpu_part:
+        return 'cortex-a72' + arch_info
+    elif '0xc08' == cpu_part:
+        return 'beaglebone'
+    elif machine == 'armv7l':
+        log.warning(
+            'WARNING: Please be advised that this device (CPU part = %s) is not officially supported by Picovoice. '
+            'Falling back to the armv6-based (Raspberry Pi Zero) library. This is not tested nor optimal.' % cpu_part)
+        return 'arm11'
+    else:
+        raise NotImplementedError("Unsupported CPU: '%s'." % cpu_part)
 
 
 def _pv_platform():
@@ -62,7 +78,10 @@ _JETSON_MACHINES = {'cortex-a57-aarch64'}
 
 def pv_library_path(relative_path):
     if _PV_SYSTEM == 'Darwin':
-        return os.path.join(os.path.dirname(__file__), relative_path, 'lib/mac/x86_64/libpv_rhino.dylib')
+        if _PV_MACHINE == 'x86_64':
+            return os.path.join(os.path.dirname(__file__), relative_path, 'lib/mac/x86_64/libpv_rhino.dylib')
+        elif _PV_MACHINE == 'arm64':
+            return os.path.join(os.path.dirname(__file__), relative_path, 'lib/mac/arm64/libpv_rhino.dylib')
     elif _PV_SYSTEM == 'Linux':
         if _PV_MACHINE == 'x86_64':
             return os.path.join(os.path.dirname(__file__), relative_path, 'lib/linux/x86_64/libpv_rhino.so')
@@ -81,9 +100,8 @@ def pv_library_path(relative_path):
     elif _PV_SYSTEM == 'Windows':
         return os.path.join(os.path.dirname(__file__), relative_path, 'lib/windows/amd64/libpv_rhino.dll')
 
-    raise NotImplementedError('Unsupported platform.')
+    raise NotImplementedError("Unsupported platform ('%s', '%s').", _PV_SYSTEM, _PV_MACHINE)
 
 
 def pv_model_path(relative_path):
     return os.path.join(os.path.dirname(__file__), relative_path, 'lib/common/rhino_params.pv')
-
